@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2003, 2004, 2005  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2003-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,17 +21,21 @@
 
 */
 
-#include "igraph.h"
-#include "memory.h"
+#include "igraph_adjlist.h"
+#include "igraph_memory.h"
+#include "igraph_interface.h"
+#include "igraph_interrupt_internal.h"
+#include "config.h"
 
 #include <string.h>   /* memset */
+#include <stdio.h>
 
 /**
  * \section about_adjlists
  * <para>Sometimes it is easier to work with a graph which is in
  * adjacency list format: a list of vectors; each vector contains the
- * neighbor vertices or adjacent edges of a given vertex. Typically,
- * this representation is good if we need to iterate over the neigbors
+ * neighbor vertices or incident edges of a given vertex. Typically,
+ * this representation is good if we need to iterate over the neighbors
  * of all vertices many times. E.g. when finding the shortest paths
  * between every pairs of vertices or calculating closeness centrality
  * for all the vertices.</para>
@@ -61,12 +65,15 @@
  * vertices), but you don't know how many vertices you will visit
  * during the computation.
  * </para>
- * 
+ *
+ * <para>
+ * \example examples/simple/adjlist.c
+ * </para>
  */
 
 /**
  * \function igraph_adjlist_init
- * Initialize an adjacency list of vertices
+ * Initialize an adjacency list of vertices from a given graph
  * 
  * Create a list of vectors containing the neighbors of all vertices
  * in a graph. The adjacency list is independent of the graph after
@@ -112,6 +119,38 @@ int igraph_adjlist_init(const igraph_t *graph, igraph_adjlist_t *al,
   return 0;
 }
 
+/**
+ * \function igraph_adjlist_init_empty
+ * Initialize an empty adjacency list
+ * 
+ * Creates a list of vectors, one for each vertex. This is useful when you
+ * are \em constructing a graph using an adjacency list representation as
+ * it does not require your graph to exist yet.
+ * \param no_of_nodes The number of vertices
+ * \param al Pointer to an uninitialized <type>igraph_adjlist_t</type> object.
+ * \return Error code.
+ * 
+ * Time complexity: O(|V|), linear in the number of vertices.
+ */
+
+int igraph_adjlist_init_empty(igraph_adjlist_t *al, igraph_integer_t no_of_nodes) {
+  long int i;
+
+  al->length=no_of_nodes;
+  al->adjs=igraph_Calloc(al->length, igraph_vector_t);
+  if (al->adjs == 0) {
+    IGRAPH_ERROR("Cannot create adjlist view", IGRAPH_ENOMEM);
+  }
+
+  IGRAPH_FINALLY(igraph_adjlist_destroy, al);
+  for (i=0; i<al->length; i++) {
+    IGRAPH_CHECK(igraph_vector_init(&al->adjs[i], 0));
+  }
+  IGRAPH_FINALLY_CLEAN(1);
+
+  return 0;
+}
+
 /**
  * \function igraph_adjlist_init_complementer
  * Adjacency lists for the complementer graph
@@ -282,20 +321,136 @@ int igraph_adjlist_simplify(igraph_adjlist_t *al) {
   return 0;
 }
 
+int igraph_adjlist_remove_duplicate(const igraph_t *graph,
+				    igraph_adjlist_t *al) {
+  long int i;
+  long int n=al->length;
+  for (i=0; i<n; i++) {
+    igraph_vector_t *v=&al->adjs[i];
+    long int j, p=1, l=igraph_vector_size(v);
+    for (j=1; j<l; j++) {
+      long int e=VECTOR(*v)[j];
+      /* Non-loop edges, and one end of loop edges are fine. */
+      /* We use here, that the vector is sorted and we also keep it sorted */
+      if (e != i || VECTOR(*v)[j-1] != e) {
+	VECTOR(*v)[p++] = e;
+      }
+    }
+    igraph_vector_resize(v, p);
+  }
+  
+  return 0;
+}
+
+#ifndef USING_R
+int igraph_adjlist_print(const igraph_adjlist_t *al, FILE *outfile) {
+  long int i;
+  long int n=al->length;
+  for (i=0; i<n; i++) {
+    igraph_vector_t *v=&al->adjs[i];
+    igraph_vector_print(v);
+  }
+  return 0;
+}
+#endif
+
+int igraph_adjedgelist_remove_duplicate(const igraph_t *graph, 
+					igraph_inclist_t *al) {
+    IGRAPH_WARNING("igraph_adjedgelist_remove_duplicate() is deprecated, use "
+                   "igraph_inclist_remove_duplicate() instead");
+    return igraph_inclist_remove_duplicate(graph, al);
+}
+
+#ifndef USING_R
+int igraph_adjedgelist_print(const igraph_inclist_t *al, FILE *outfile) {
+    IGRAPH_WARNING("igraph_adjedgelist_print() is deprecated, use "
+                   "igraph_inclist_print() instead");
+    return igraph_inclist_print(al, outfile);
+}
+#endif
+
 /**
  * \function igraph_adjedgelist_init
- * Initialize an adjacency list of edges
+ * Initialize an incidence list of edges
+ *
+ * This function was superseded by \ref igraph_inclist_init() in igraph 0.6.
+ * Please use \ref igraph_inclist_init() instead of this function.
+ *
+ * </para><para>
+ * Deprecated in version 0.6.
+ */
+int igraph_adjedgelist_init(const igraph_t *graph, 
+			                igraph_inclist_t *il, 
+			                igraph_neimode_t mode) {
+    IGRAPH_WARNING("igraph_adjedgelist_init() is deprecated, use "
+                   "igraph_inclist_init() instead");
+    return igraph_inclist_init(graph, il, mode);
+}
+
+/**
+ * \function igraph_adjedgelist_destroy
+ * Frees all memory allocated for an incidence list.
+ *
+ * This function was superseded by \ref igraph_inclist_destroy() in igraph 0.6.
+ * Please use \ref igraph_inclist_destroy() instead of this function.
+ *
+ * </para><para>
+ * Deprecated in version 0.6.
+ */
+void igraph_adjedgelist_destroy(igraph_inclist_t *il) {
+    IGRAPH_WARNING("igraph_adjedgelist_destroy() is deprecated, use "
+                   "igraph_inclist_destroy() instead");
+    igraph_inclist_destroy(il);
+}
+
+int igraph_inclist_remove_duplicate(const igraph_t *graph, 
+					igraph_inclist_t *al) {
+  long int i;
+  long int n=al->length;
+  for (i=0; i<n; i++) {
+    igraph_vector_t *v=&al->incs[i];
+    long int j, p=1, l=igraph_vector_size(v);
+    for (j=1; j<l; j++) {
+      long int e=VECTOR(*v)[j];
+      /* Non-loop edges and one end of loop edges are fine. */
+      /* We use here, that the vector is sorted and we also keep it sorted */
+      if (IGRAPH_FROM(graph, e) != IGRAPH_TO(graph, e) ||
+	  VECTOR(*v)[j-1] != e) {
+	VECTOR(*v)[p++] = e;
+      }
+    }
+    igraph_vector_resize(v, p);
+  }
+  
+  return 0;
+}
+
+#ifndef USING_R
+int igraph_inclist_print(const igraph_inclist_t *al, FILE *outfile) {
+  long int i;
+  long int n=al->length;
+  for (i=0; i<n; i++) {
+    igraph_vector_t *v=&al->incs[i];
+    igraph_vector_print(v);
+  }
+  return 0;
+}
+#endif
+
+/**
+ * \function igraph_inclist_init
+ * Initialize an incidence list of edges
  * 
- * Create a list of vectors containing the adjacent edges for all
- * vertices. The adjacency list is independent of the graph after
+ * Create a list of vectors containing the incident edges for all
+ * vertices. The incidence list is independent of the graph after
  * creation, subsequent changes of the graph object do not update the
- * adjacency list, and changes to the adjacency list do no update the
+ * incidence list, and changes to the incidence list do not update the
  * graph.
  * \param graph The input graph.
- * \param ael Pointer to an uninitialized adjcency list.
+ * \param il Pointer to an uninitialized incidence list.
  * \param mode Constant specifying whether incoming edges
  *   (<code>IGRAPH_IN</code>), outgoing edges (<code>IGRAPH_OUT</code>) or
- *   both (<code>IGRAPH_ALL</code>) to include in the adjacency lists
+ *   both (<code>IGRAPH_ALL</code>) to include in the incidence lists
  *   of directed graphs. It is ignored for undirected graphs.
  * \return Error code.
  * 
@@ -303,28 +458,28 @@ int igraph_adjlist_simplify(igraph_adjlist_t *al) {
  * edges.
  */
 
-int igraph_adjedgelist_init(const igraph_t *graph, 
-			      igraph_adjedgelist_t *ael, 
+int igraph_inclist_init(const igraph_t *graph, 
+			      igraph_inclist_t *il, 
 			      igraph_neimode_t mode) {
   long int i;
 
   if (mode != IGRAPH_IN && mode != IGRAPH_OUT && mode != IGRAPH_ALL) {
-    IGRAPH_ERROR("Cannot create adjedgelist view", IGRAPH_EINVMODE);
+    IGRAPH_ERROR("Cannot create incidence list view", IGRAPH_EINVMODE);
   }
 
   if (!igraph_is_directed(graph)) { mode=IGRAPH_ALL; }
 
-  ael->length=igraph_vcount(graph);
-  ael->adjs=igraph_Calloc(ael->length, igraph_vector_t);
-  if (ael->adjs == 0) {
-    IGRAPH_ERROR("Cannot create adjedgelist view", IGRAPH_ENOMEM);
+  il->length=igraph_vcount(graph);
+  il->incs=igraph_Calloc(il->length, igraph_vector_t);
+  if (il->incs == 0) {
+    IGRAPH_ERROR("Cannot create incidence list view", IGRAPH_ENOMEM);
   }
 
-  IGRAPH_FINALLY(igraph_adjlist_destroy, ael);  
-  for (i=0; i<ael->length; i++) {
+  IGRAPH_FINALLY(igraph_inclist_destroy, il);  
+  for (i=0; i<il->length; i++) {
     IGRAPH_ALLOW_INTERRUPTION();
-    IGRAPH_CHECK(igraph_vector_init(&ael->adjs[i], 0));
-    IGRAPH_CHECK(igraph_adjacent(graph, &ael->adjs[i], i, mode));
+    IGRAPH_CHECK(igraph_vector_init(&il->incs[i], 0));
+    IGRAPH_CHECK(igraph_incident(graph, &il->incs[i], i, mode));
   }
   
   IGRAPH_FINALLY_CLEAN(1);
@@ -332,23 +487,55 @@ int igraph_adjedgelist_init(const igraph_t *graph,
 }
 
 /**
- * \function igraph_adjedgelist_destroy
- * Destroy
+ * \function igraph_inclist_init_empty
+ * \brief Initialize an incidence list corresponding to an empty graph.
  * 
- * Free all memory allocated for an adjacency list.
- * \param eal The adjcency list to destroy.
+ * This function essentially creates a list of empty vectors that may
+ * be treated as an incidence list for a graph with a given number of
+ * vertices.
+ *
+ * \param il Pointer to an uninitialized incidence list.
+ * \param n  The number of vertices in the incidence list.
+ * \return Error code.
+ * 
+ * Time complexity: O(|V|), linear in the number of vertices.
+ */
+
+int igraph_inclist_init_empty(igraph_inclist_t *il, igraph_integer_t n) {
+  long int i;
+
+  il->length=n;
+  il->incs=igraph_Calloc(il->length, igraph_vector_t);
+  if (il->incs == 0) {
+    IGRAPH_ERROR("Cannot create incidence list view", IGRAPH_ENOMEM);
+  }
+
+  IGRAPH_FINALLY(igraph_inclist_destroy, il);  
+  for (i=0; i<n; i++) {
+    IGRAPH_CHECK(igraph_vector_init(&il->incs[i], 0));
+  }
+  
+  IGRAPH_FINALLY_CLEAN(1);
+  return 0;
+}
+
+/**
+ * \function igraph_inclist_destroy
+ * Frees all memory allocated for an incidence list.
+ *
+ * \param eal The incidence list to destroy.
  * 
  * Time complexity: depends on memory management.
  */
 
-void igraph_adjedgelist_destroy(igraph_adjedgelist_t *ael) {
+void igraph_inclist_destroy(igraph_inclist_t *il) {
   long int i;
-  for (i=0; i<ael->length; i++) {
+  for (i=0; i<il->length; i++) {
     /* This works if some igraph_vector_t's are 0, because igraph_vector_destroy can
        handle this. */
-    igraph_vector_destroy(&ael->adjs[i]);
+    igraph_vector_destroy(&il->incs[i]);
   }
-  igraph_Free(ael->adjs);
+  igraph_Free(il->incs);
 }
 
 /**
@@ -367,7 +554,7 @@ void igraph_adjedgelist_destroy(igraph_adjedgelist_t *ael) {
  *   (<code>IGRAPH_ALL</code>) are considered. It is ignored for
  *   undirected graphs.
  * \param simplify Constant, it gives whether to simplify the vectors
- *   in the adjacency list (<code>IGRAPH_SIMPLIFY</code>) ot not
+ *   in the adjacency list (<code>IGRAPH_SIMPLIFY</code>) or not
  *   (<code>IGRAPH_DONT_SIMPLIFY</code>).
  * \return Error code.
  * 
@@ -456,14 +643,55 @@ igraph_vector_t *igraph_lazy_adjlist_get_real(igraph_lazy_adjlist_t *al,
 
 /**
  * \function igraph_lazy_adjedgelist_init
- * Constructor
+ * Initializes a lazy incidence list of edges
+ *
+ * This function was superseded by \ref igraph_lazy_inclist_init() in igraph 0.6.
+ * Please use \ref igraph_lazy_inclist_init() instead of this function.
+ *
+ * </para><para>
+ * Deprecated in version 0.6.
+ */
+int igraph_lazy_adjedgelist_init(const igraph_t *graph, 
+			                igraph_lazy_inclist_t *il, 
+			                igraph_neimode_t mode) {
+    IGRAPH_WARNING("igraph_lazy_adjedgelist_init() is deprecated, use "
+                   "igraph_lazy_inclist_init() instead");
+    return igraph_lazy_inclist_init(graph, il, mode);
+}
+
+/**
+ * \function igraph_lazy_adjedgelist_destroy
+ * Frees all memory allocated for an incidence list.
+ *
+ * This function was superseded by \ref igraph_lazy_inclist_destroy() in igraph 0.6.
+ * Please use \ref igraph_lazy_inclist_destroy() instead of this function.
+ *
+ * </para><para>
+ * Deprecated in version 0.6.
+ */
+void igraph_lazy_adjedgelist_destroy(igraph_lazy_inclist_t *il) {
+    IGRAPH_WARNING("igraph_lazy_adjedgelist_destroy() is deprecated, use "
+                   "igraph_lazy_inclist_destroy() instead");
+    igraph_lazy_inclist_destroy(il);
+}
+
+igraph_vector_t *igraph_lazy_adjedgelist_get_real(igraph_lazy_adjedgelist_t *il,
+						    igraph_integer_t pno) {
+    IGRAPH_WARNING("igraph_lazy_adjedgelist_get_real() is deprecated, use "
+                   "igraph_lazy_inclist_get_real() instead");
+    return igraph_lazy_inclist_get_real(il, pno);
+}
+
+/**
+ * \function igraph_lazy_inclist_init
+ * Initializes a lazy incidence list of edges
  * 
- * Create a lazy adjacency list for edges. This function only
- * allocates some memory for storing the vectors of an adjacency list,
- * but the adjacent edges are not queried, only when \ref
- * igraph_lazy_adjedgelist_get() is called.
+ * Create a lazy incidence list for edges. This function only
+ * allocates some memory for storing the vectors of an incidence list,
+ * but the incident edges are not queried, only when \ref
+ * igraph_lazy_inclist_get() is called.
  * \param graph The input graph.
- * \param al Pointer to an uninitialized adjacency list.
+ * \param al Pointer to an uninitialized incidence list.
  * \param mode Constant, it gives whether incoming edges
  *   (<code>IGRAPH_IN</code>), outgoing edges
  *   (<code>IGRPAH_OUT</code>) or both types of edges
@@ -472,15 +700,15 @@ igraph_vector_t *igraph_lazy_adjlist_get_real(igraph_lazy_adjlist_t *al,
  * \return Error code.
  * 
  * Time complexity: O(|V|), the number of vertices, possibly. But it
- * also depends on the underlying memory management too.
+ * also depends on the underlying memory management.
  */
 
-int igraph_lazy_adjedgelist_init(const igraph_t *graph,
-				   igraph_lazy_adjedgelist_t *al,
+int igraph_lazy_inclist_init(const igraph_t *graph,
+				   igraph_lazy_inclist_t *al,
 				   igraph_neimode_t mode) {
 
   if (mode != IGRAPH_IN && mode != IGRAPH_OUT && mode != IGRAPH_ALL) {
-    IGRAPH_ERROR("Cannot create adjlist view", IGRAPH_EINVMODE);
+    IGRAPH_ERROR("Cannot create lazy incidence list view", IGRAPH_EINVMODE);
   }
   
   if (!igraph_is_directed(graph)) { mode=IGRAPH_ALL; }
@@ -489,9 +717,9 @@ int igraph_lazy_adjedgelist_init(const igraph_t *graph,
   al->graph=graph;
   
   al->length=igraph_vcount(graph);
-  al->adjs=igraph_Calloc(al->length, igraph_vector_t*);
-  if (al->adjs == 0) {
-    IGRAPH_ERROR("Cannot create lazy adjedgelist view", IGRAPH_ENOMEM);
+  al->incs=igraph_Calloc(al->length, igraph_vector_t*);
+  if (al->incs == 0) {
+    IGRAPH_ERROR("Cannot create lazy incidence list view", IGRAPH_ENOMEM);
   }
 
   return 0;
@@ -499,44 +727,44 @@ int igraph_lazy_adjedgelist_init(const igraph_t *graph,
 }
 
 /**
- * \function igraph_lazy_adjedgelist_destroy
- * Deallocate memory
+ * \function igraph_lazy_inclist_destroy
+ * Deallocates memory
  * 
- * Free all allocated memory for a lazy edge adjacency list.
- * \param al The adjacency list to deallocate.
+ * Frees all allocated memory for a lazy incidence list.
+ * \param al The incidence list to deallocate.
  * 
  * Time complexity: depends on memory management.
  */
 
-void igraph_lazy_adjedgelist_destroy(igraph_lazy_adjedgelist_t *al) {
-  long int i, n=al->length;
+void igraph_lazy_inclist_destroy(igraph_lazy_inclist_t *il) {
+  long int i, n=il->length;
   for (i=0; i<n; i++) {
-    if (al->adjs[i] != 0) {
-      igraph_vector_destroy(al->adjs[i]);
-      igraph_Free(al->adjs[i]);
+    if (il->incs[i] != 0) {
+      igraph_vector_destroy(il->incs[i]);
+      igraph_Free(il->incs[i]);
     }
   }
-  igraph_Free(al->adjs);
+  igraph_Free(il->incs);
 }
 
-igraph_vector_t *igraph_lazy_adjedgelist_get_real(igraph_lazy_adjedgelist_t *al,
+igraph_vector_t *igraph_lazy_inclist_get_real(igraph_lazy_inclist_t *il,
 						    igraph_integer_t pno) {
   long int no=pno;
   int ret;
-  if (al->adjs[no] == 0) {
-    al->adjs[no] = igraph_Calloc(1, igraph_vector_t);
-    if (al->adjs[no] == 0) {
-      igraph_error("Lazy adjedgelist failed", __FILE__, __LINE__, 
+  if (il->incs[no] == 0) {
+    il->incs[no] = igraph_Calloc(1, igraph_vector_t);
+    if (il->incs[no] == 0) {
+      igraph_error("Lazy incidence list query failed", __FILE__, __LINE__, 
 		   IGRAPH_ENOMEM);
     }
-    ret=igraph_vector_init(al->adjs[no], 0);
+    ret=igraph_vector_init(il->incs[no], 0);
     if (ret != 0) {
       igraph_error("", __FILE__, __LINE__, ret);
     }
-    ret=igraph_adjacent(al->graph, al->adjs[no], no, al->mode);
+    ret=igraph_incident(il->graph, il->incs[no], no, il->mode);
     if (ret != 0) {
       igraph_error("", __FILE__, __LINE__, ret);
     }
   }
-  return al->adjs[no];
+  return il->incs[no];
 }
