@@ -1,8 +1,8 @@
  /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2005  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2005-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,10 +21,20 @@
 
 */
 
-#include "igraph.h"
-#include "memory.h"
-#include "random.h"
+#include "igraph_cliques.h"
+#include "igraph_memory.h"
+#include "igraph_random.h"
+#include "igraph_constants.h"
+#include "igraph_adjlist.h"
+#include "igraph_interrupt_internal.h"
+#include "igraph_interface.h"
+#include "igraph_progress.h"
+#include "igraph_stack.h"
+#include "igraph_types_internal.h"
+#include "config.h"
 
+#include <assert.h>
+#include <string.h>    /* memset */
 
 void igraph_i_cliques_free_res(igraph_vector_ptr_t *res) {
   long i, n;
@@ -270,7 +280,7 @@ int igraph_i_cliques(const igraph_t *graph, igraph_vector_ptr_t *res,
   igraph_free(member_storage);
   igraph_free(new_member_storage);
   igraph_vector_destroy(&neis);
-  IGRAPH_FINALLY_CLEAN(4); /* 2 here, +1 is igraph_i_cliques_free_res */
+  IGRAPH_FINALLY_CLEAN(4); /* 3 here, +1 is igraph_i_cliques_free_res */
   
   return 0;
 }
@@ -286,6 +296,14 @@ int igraph_i_cliques(const igraph_t *graph, igraph_vector_ptr_t *res,
  * If you are only interested in the size of the largest clique in the graph,
  * use \ref igraph_clique_number() instead.
  *
+ * </para><para>The current implementation of this function searches
+ * for maximal independent vertex sets (see \ref
+ * igraph_maximal_independent_vertex_sets()) in the complementer graph
+ * using the algorithm published in: 
+ * S. Tsukiyama, M. Ide, H. Ariyoshi and I. Shirawaka. A new algorithm
+ * for generating all the maximal independent sets. SIAM J Computing,
+ * 6:505--517, 1977.
+ *
  * \param graph The input graph.
  * \param res Pointer to a pointer vector, the result will be stored
  *   here, ie. \c res will contain pointers to \c igraph_vector_t
@@ -301,45 +319,29 @@ int igraph_i_cliques(const igraph_t *graph, igraph_vector_ptr_t *res,
  * \sa \ref igraph_largest_cliques() and \ref igraph_clique_number().
  *
  * Time complexity: TODO
+ * 
+ * \example examples/simple/igraph_cliques.c
  */
 int igraph_cliques(const igraph_t *graph, igraph_vector_ptr_t *res,
                    igraph_integer_t min_size, igraph_integer_t max_size) {
   return igraph_i_cliques(graph, res, min_size, max_size, 0);
 }
 
+typedef int(*igraph_i_maximal_clique_func_t)(const igraph_vector_t*, void*, igraph_bool_t*);
+typedef struct {
+    igraph_vector_ptr_t* result;
+    igraph_integer_t min_size;
+    igraph_integer_t max_size;
+} igraph_i_maximal_clique_data_t;
+
+int igraph_i_maximal_cliques(const igraph_t *graph, igraph_i_maximal_clique_func_t func, void* data);
+
 int igraph_i_maximal_or_largest_cliques_or_indsets(const igraph_t *graph,
                                         igraph_vector_ptr_t *res,
                                         igraph_integer_t *clique_number,
                                         igraph_bool_t keep_only_largest,
 					igraph_bool_t complementer);
 
-/**
- * \function igraph_largest_cliques
- * \brief Finds the largest clique(s) in a graph.
- * 
- * </para><para>
- * A clique is largest (quite intuitively) if there is no other clique
- * in the graph which contains more vertices. 
- * 
- * </para><para>
- * Note that this is not neccessarily the same as a maximal clique,
- * ie. the largest cliques are always maximal but a maximal clique is
- * not always largest.
- * \param graph The input graph.
- * \param res Pointer to an initialized pointer vector, the result
- *        will be stored here. It will be resized as needed.
- * \return Error code.
- * 
- * \sa \ref igraph_cliques(), \ref igraph_maximal_cliques()
- * 
- * Time complexity: TODO.
- */
-
-int igraph_largest_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
-  return igraph_i_maximal_or_largest_cliques_or_indsets(graph, res, 0, 1, 1);
-}
-
-
 /**
  * \function igraph_independent_vertex_sets
  * \brief Find all independent vertex sets in a graph
@@ -352,6 +354,13 @@ int igraph_largest_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
  * If you are interested in the size of the largest independent vertex set,
  * use \ref igraph_independence_number() instead.
  *
+ * </para><para>
+ * The current implementation was ported to igraph from the Very Nauty Graph
+ * Library by Keith Briggs and uses the algorithm from the paper
+ * S. Tsukiyama, M. Ide, H. Ariyoshi and I. Shirawaka. A new algorithm
+ * for generating all the maximal independent sets. SIAM J Computing,
+ * 6:505--517, 1977.
+ *
  * \param graph The input graph.
  * \param res Pointer to a pointer vector, the result will be stored
  *   here, ie. \c res will contain pointers to \c igraph_vector_t
@@ -368,6 +377,8 @@ int igraph_largest_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
  * \ref igraph_independence_number().
  *
  * Time complexity: TODO
+ * 
+ * \example examples/simple/igraph_independent_sets.c
  */
 int igraph_independent_vertex_sets(const igraph_t *graph,
 				   igraph_vector_ptr_t *res,
@@ -383,6 +394,14 @@ int igraph_independent_vertex_sets(const igraph_t *graph,
  * </para><para>
  * An independent vertex set is largest if there is no other
  * independent vertex set with more vertices in the graph.
+ *
+ * </para><para>
+ * The current implementation was ported to igraph from the Very Nauty Graph
+ * Library by Keith Briggs and uses the algorithm from the paper
+ * S. Tsukiyama, M. Ide, H. Ariyoshi and I. Shirawaka. A new algorithm
+ * for generating all the maximal independent sets. SIAM J Computing,
+ * 6:505--517, 1977.
+ *
  * \param graph The input graph.
  * \param res Pointer to a pointer vector, the result will be stored
  *     here. It will be resized as needed.
@@ -450,7 +469,10 @@ int igraph_i_maximal_independent_vertex_sets_backtrack(const igraph_t *graph,
           IGRAPH_CHECK(igraph_vector_ptr_push_back(res, vec));
         } else if (size == clqdata->largest_set_size) {
           IGRAPH_CHECK(igraph_vector_ptr_push_back(res, vec));
-        }
+        } else {
+		  igraph_vector_destroy(vec);
+		  free(vec);
+		}
       }
       IGRAPH_FINALLY_CLEAN(1);
     } else {
@@ -633,6 +655,13 @@ int igraph_maximal_independent_vertex_sets(const igraph_t *graph,
  * The independence number of a graph is the cardinality of the largest
  * independent vertex set.
  *
+ * </para><para>
+ * The current implementation was ported to igraph from the Very Nauty Graph
+ * Library by Keith Briggs and uses the algorithm from the paper
+ * S. Tsukiyama, M. Ide, H. Ariyoshi and I. Shirawaka. A new algorithm
+ * for generating all the maximal independent sets. SIAM J Computing,
+ * 6:505--517, 1977.
+ *
  * \param graph The input graph.
  * \param no The independence number will be returned to the \c
  *   igraph_integer_t pointed by this variable.
@@ -688,35 +717,185 @@ int igraph_independence_number(const igraph_t *graph, igraph_integer_t *no) {
   return 0;
 }
 
+/*************************************************************************/
+/* MAXIMAL CLIQUES, LARGEST CLIQUES                                      */
+/*************************************************************************/
+
+int igraph_i_maximal_cliques_store_max_size(const igraph_vector_t* clique, void* data,
+    igraph_bool_t* cont) {
+  igraph_integer_t* result = (igraph_integer_t*)data;
+  if (*result < igraph_vector_size(clique))
+    *result = igraph_vector_size(clique);
+  return IGRAPH_SUCCESS;
+}
+
+int igraph_i_maximal_cliques_store(const igraph_vector_t* clique, void* data, igraph_bool_t* cont) {
+  igraph_vector_ptr_t* result = (igraph_vector_ptr_t*)data;
+  igraph_vector_t* vec;
+
+  vec = igraph_Calloc(1, igraph_vector_t);
+  if (vec == 0)
+    IGRAPH_ERROR("cannot allocate memory for storing next clique", IGRAPH_ENOMEM);
+
+  IGRAPH_CHECK(igraph_vector_copy(vec, clique));
+  IGRAPH_CHECK(igraph_vector_ptr_push_back(result, vec));
+
+  return IGRAPH_SUCCESS;
+}
+
+int igraph_i_maximal_cliques_store_size_check(const igraph_vector_t* clique, void* data_, igraph_bool_t* cont) {
+  igraph_i_maximal_clique_data_t* data = (igraph_i_maximal_clique_data_t*)data_;
+  igraph_vector_t* vec;
+  igraph_integer_t size = igraph_vector_size(clique);
+
+  if (size < data->min_size || size > data->max_size)
+    return IGRAPH_SUCCESS;
+
+  vec = igraph_Calloc(1, igraph_vector_t);
+  if (vec == 0)
+    IGRAPH_ERROR("cannot allocate memory for storing next clique", IGRAPH_ENOMEM);
+
+  IGRAPH_CHECK(igraph_vector_copy(vec, clique));
+  IGRAPH_CHECK(igraph_vector_ptr_push_back(data->result, vec));
+
+  return IGRAPH_SUCCESS;
+}
+
+int igraph_i_largest_cliques_store(const igraph_vector_t* clique, void* data, igraph_bool_t* cont) {
+  igraph_vector_ptr_t* result = (igraph_vector_ptr_t*)data;
+  igraph_vector_t* vec;
+  long int i, n;
+
+  /* Is the current clique at least as large as the others that we have found? */
+  if (!igraph_vector_ptr_empty(result)) {
+    n = igraph_vector_size(clique);
+    if (n < igraph_vector_size(VECTOR(*result)[0]))
+      return IGRAPH_SUCCESS;
+
+    if (n > igraph_vector_size(VECTOR(*result)[0])) {
+      for (i = 0; i < igraph_vector_ptr_size(result); i++)
+        igraph_vector_destroy(VECTOR(*result)[i]);
+      igraph_vector_ptr_free_all(result);
+      igraph_vector_ptr_resize(result, 0);
+    }
+  }
+
+  vec = igraph_Calloc(1, igraph_vector_t);
+  if (vec == 0)
+    IGRAPH_ERROR("cannot allocate memory for storing next clique", IGRAPH_ENOMEM);
+
+  IGRAPH_CHECK(igraph_vector_copy(vec, clique));
+  IGRAPH_CHECK(igraph_vector_ptr_push_back(result, vec));
+
+  return IGRAPH_SUCCESS;
+}
+
+/**
+ * \function igraph_largest_cliques
+ * \brief Finds the largest clique(s) in a graph.
+ * 
+ * </para><para>
+ * A clique is largest (quite intuitively) if there is no other clique
+ * in the graph which contains more vertices. 
+ * 
+ * </para><para>
+ * Note that this is not necessarily the same as a maximal clique,
+ * ie. the largest cliques are always maximal but a maximal clique is
+ * not always largest.
+ *
+ * </para><para>The current implementation of this function searches
+ * for maximal cliques using \ref igraph_maximal_cliques() and drops
+ * those that are not the largest.
+ *
+ * </para><para>The implementation of this function changed between
+ * igraph 0.5 and 0.6, so the order of the cliques and the order of
+ * vertices within the cliques will almost surely be different between
+ * these two versions.
+ *
+ * \param graph The input graph.
+ * \param res Pointer to an initialized pointer vector, the result
+ *        will be stored here. It will be resized as needed. Note that
+ *        vertices of a clique may be returned in arbitrary order.
+ * \return Error code.
+ * 
+ * \sa \ref igraph_cliques(), \ref igraph_maximal_cliques()
+ * 
+ * Time complexity: O(3^(|V|/3)) worst case.
+ */
+
+int igraph_largest_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
+  igraph_vector_ptr_clear(res);
+  IGRAPH_FINALLY(igraph_i_cliques_free_res, res);
+  IGRAPH_CHECK(igraph_i_maximal_cliques(graph, &igraph_i_largest_cliques_store, (void*)res));
+  IGRAPH_FINALLY_CLEAN(1);
+  return IGRAPH_SUCCESS;
+}
+
+
 /**
  * \function igraph_maximal_cliques
  * \brief Find all maximal cliques of a graph
  *
  * </para><para>
- * A maximal clique is a clique which
- * can't be extended any more by adding a new vertex to it. This is actually
- * implemented by looking for a maximal independent vertex set in the
- * complementer of the graph.
- * 
+ * A maximal clique is a clique which can't be extended any more by adding a
+ * new vertex to it.
+ *
  * </para><para>
  * If you are only interested in the size of the largest clique in the graph,
  * use \ref igraph_clique_number() instead.
  *
+ * </para><para>
+ * The current implementation uses the Bron-Kerbosch algorithm to find
+ * the maximal cliques, see: C. Bron and J. Kerbosch. Algorithm 457:
+ * finding all cliques of an undirected graph. Communications of the ACM
+ * 16(9):575-577, 1973.
+ *
+ * </para><para>The implementation of this function changed between
+ * igraph 0.5 and 0.6, so the order of the cliques and the order of
+ * vertices within the cliques will almost surely be different between
+ * these two versions.
+ *
  * \param graph The input graph.
  * \param res Pointer to a pointer vector, the result will be stored
  *   here, ie. \c res will contain pointers to \c igraph_vector_t
  *   objects which contain the indices of vertices involved in a clique.
  *   The pointer vector will be resized if needed but note that the
- *   objects in the pointer vector will not be freed.
+ *   objects in the pointer vector will not be freed. Note that vertices
+ *   of a clique may be returned in arbitrary order.
+ * \param min_size Integer giving the minimum size of the cliques to be
+ *   returned. If negative or zero, no lower bound will be used.
+ * \param max_size Integer giving the maximum size of the cliques to be
+ *   returned. If negative or zero, no upper bound will be used.
  * \return Error code.
  *
  * \sa \ref igraph_maximal_independent_vertex_sets(), \ref
  * igraph_clique_number() 
  * 
- * Time complexity: TODO.
+ * Time complexity: O(3^(|V|/3)) worst case.
+ * 
+ * \example examples/simple/igraph_maximal_cliques.c
  */
-int igraph_maximal_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
-  return igraph_i_maximal_or_largest_cliques_or_indsets(graph, res, 0, 0, 1);
+int igraph_maximal_cliques(const igraph_t *graph, igraph_vector_ptr_t *res,
+        igraph_integer_t min_size, igraph_integer_t max_size) {
+  igraph_i_maximal_clique_data_t data;
+
+  igraph_vector_ptr_clear(res);
+  IGRAPH_FINALLY(igraph_i_cliques_free_res, res);
+
+  if (min_size <= 0 && (max_size <= 0 || max_size >= igraph_vcount(graph))) {
+    /* Search without size limits */
+    IGRAPH_CHECK(igraph_i_maximal_cliques(graph, &igraph_i_maximal_cliques_store, (void*)res));
+  } else {
+    /* Size-limited search */
+    data.result = res;
+    data.min_size = (min_size <= 0 ? 1 : min_size);
+    data.max_size = (max_size <= 0 ? igraph_vcount(graph) : max_size);
+    IGRAPH_CHECK(igraph_i_maximal_cliques(graph, &igraph_i_maximal_cliques_store_size_check, (void*)&data));
+  }
+
+  IGRAPH_FINALLY_CLEAN(1);
+
+  return IGRAPH_SUCCESS;
 }
 
 /**
@@ -733,10 +912,245 @@ int igraph_maximal_cliques(const igraph_t *graph, igraph_vector_ptr_t *res) {
  * 
  * \sa \ref igraph_cliques(), \ref igraph_largest_cliques().
  * 
- * Time complexity: TODO.
+ * Time complexity: O(3^(|V|/3)) worst case.
  */
 int igraph_clique_number(const igraph_t *graph, igraph_integer_t *no) {
-  return igraph_i_maximal_or_largest_cliques_or_indsets(graph, 0, no, 0, 1);
+  *no = 0;
+  return igraph_i_maximal_cliques(graph, &igraph_i_maximal_cliques_store_max_size, (void*)no);
+}
+
+typedef struct {
+  igraph_vector_t cand;
+  igraph_vector_t fini;
+  igraph_vector_t cand_filtered;
+} igraph_i_maximal_cliques_stack_frame;
+
+void igraph_i_maximal_cliques_stack_frame_destroy(igraph_i_maximal_cliques_stack_frame *frame) {
+  igraph_vector_destroy(&frame->cand);
+  igraph_vector_destroy(&frame->fini);
+  igraph_vector_destroy(&frame->cand_filtered);
+}
+
+void igraph_i_maximal_cliques_stack_destroy(igraph_stack_ptr_t *stack) {
+  igraph_i_maximal_cliques_stack_frame *frame;
+
+  while (!igraph_stack_ptr_empty(stack)) {
+    frame = (igraph_i_maximal_cliques_stack_frame*)igraph_stack_ptr_pop(stack);
+    igraph_i_maximal_cliques_stack_frame_destroy(frame);
+    free(frame);
+  }
+
+  igraph_stack_ptr_destroy(stack);
+}
+
+int igraph_i_maximal_cliques(const igraph_t *graph, igraph_i_maximal_clique_func_t func, void* data) {
+  long int i, j, k, l;
+  igraph_integer_t no_of_nodes, nodes_to_check, nodes_done;
+  igraph_integer_t best_cand = 0, best_cand_degree = 0, best_fini_cand_degree;
+  igraph_adjlist_t adj_list;
+  igraph_stack_ptr_t stack;
+  igraph_i_maximal_cliques_stack_frame frame, *new_frame_ptr;
+  igraph_vector_t clique, new_cand, new_fini, cn, best_cand_nbrs, best_fini_cand_nbrs;
+  igraph_bool_t cont = 1;
+  int assret;
+
+  if (igraph_is_directed(graph))
+    IGRAPH_WARNING("directionality of edges is ignored for directed graphs");
+
+  no_of_nodes = igraph_vcount(graph);
+  if (no_of_nodes == 0)
+    return IGRAPH_SUCCESS;
+
+  /* Construct an adjacency list representation */
+  IGRAPH_CHECK(igraph_adjlist_init(graph, &adj_list, IGRAPH_ALL));
+  IGRAPH_FINALLY(igraph_adjlist_destroy, &adj_list);
+  IGRAPH_CHECK(igraph_adjlist_simplify(&adj_list));
+
+  /* Initialize stack */
+  IGRAPH_CHECK(igraph_stack_ptr_init(&stack, 0));
+  IGRAPH_FINALLY(igraph_i_maximal_cliques_stack_destroy, &stack);
+
+  /* Create the initial (empty) clique */
+  IGRAPH_VECTOR_INIT_FINALLY(&clique, 0);
+
+  /* Initialize new_cand, new_fini, cn, best_cand_nbrs and best_fini_cand_nbrs (will be used later) */
+  IGRAPH_VECTOR_INIT_FINALLY(&new_cand, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&new_fini, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&cn, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&best_cand_nbrs, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&best_fini_cand_nbrs, 0);
+
+  /* Find the vertex with the highest degree */
+  best_cand = 0; best_cand_degree = igraph_vector_size(igraph_adjlist_get(&adj_list, 0));
+  for (i = 1; i < no_of_nodes; i++) {
+    j = igraph_vector_size(igraph_adjlist_get(&adj_list, i));
+    if (j > best_cand_degree) {
+      best_cand = i;
+      best_cand_degree = j;
+    }
+  }
+
+  /* Create the initial stack frame */
+  IGRAPH_CHECK(igraph_vector_init_seq(&frame.cand, 0, no_of_nodes-1));
+  IGRAPH_FINALLY(igraph_vector_destroy, &frame.cand);
+  IGRAPH_CHECK(igraph_vector_init(&frame.fini, 0));
+  IGRAPH_FINALLY(igraph_vector_destroy, &frame.fini);
+  IGRAPH_CHECK(igraph_vector_init(&frame.cand_filtered, 0));
+  IGRAPH_FINALLY(igraph_vector_destroy, &frame.cand_filtered);
+  IGRAPH_CHECK(igraph_vector_difference_sorted(&frame.cand,
+        igraph_adjlist_get(&adj_list, best_cand), &frame.cand_filtered));
+  IGRAPH_FINALLY_CLEAN(3);
+  IGRAPH_FINALLY(igraph_i_maximal_cliques_stack_frame_destroy, &frame);
+
+  /* TODO: frame.cand and frame.fini should be a set instead of a vector */
+
+  /* Main loop starts here */
+  nodes_to_check = igraph_vector_size(&frame.cand_filtered); nodes_done = 0;
+  while (!igraph_vector_empty(&frame.cand_filtered) || !igraph_stack_ptr_empty(&stack)) {
+    if (igraph_vector_empty(&frame.cand_filtered)) {
+      /* No candidates left to check in this stack frame, pop out the previous stack frame */
+      igraph_i_maximal_cliques_stack_frame *newframe = igraph_stack_ptr_pop(&stack);
+      igraph_i_maximal_cliques_stack_frame_destroy(&frame);
+      frame = *newframe;
+      free(newframe);
+
+      if (igraph_stack_ptr_size(&stack) == 1) {
+        /* We will be using the next candidate node in the next iteration, so we can increase
+         * nodes_done by 1 */
+        nodes_done++;
+      }
+
+      /* For efficiency reasons, we only check for interruption and show progress here */
+      IGRAPH_PROGRESS("Maximal cliques: ", 100.0 * nodes_done / nodes_to_check, NULL);
+      IGRAPH_ALLOW_INTERRUPTION();
+
+      igraph_vector_pop_back(&clique);
+      continue;
+    }
+
+    /* Try the next node in the clique */
+    i = igraph_vector_pop_back(&frame.cand_filtered);
+    IGRAPH_CHECK(igraph_vector_push_back(&clique, i));
+
+    /* Remove the node from the candidate list */
+    assret=igraph_vector_binsearch(&frame.cand, i, &j); assert(assret);
+    igraph_vector_remove(&frame.cand, j);
+
+    /* Add the node to the finished list */
+    assret = !igraph_vector_binsearch(&frame.fini, i, &j); assert(assret);
+    IGRAPH_CHECK(igraph_vector_insert(&frame.fini, j, i));
+
+    /* Create new_cand and new_fini */
+    IGRAPH_CHECK(igraph_vector_intersect_sorted(&frame.cand, igraph_adjlist_get(&adj_list, i), &new_cand));
+    IGRAPH_CHECK(igraph_vector_intersect_sorted(&frame.fini, igraph_adjlist_get(&adj_list, i), &new_fini));
+
+    /* Do we have anything more to search? */
+    if (igraph_vector_empty(&new_cand)) {
+      if (igraph_vector_empty(&new_fini)) {
+        /* We have a maximal clique here */
+        IGRAPH_CHECK(func(&clique, data, &cont));
+        if (!cont) {
+          /* The callback function requested to stop the search */
+          break;
+        }
+      }
+      igraph_vector_pop_back(&clique);
+      continue;
+    }
+    if (igraph_vector_empty(&new_fini) && igraph_vector_size(&new_cand) == 1) {
+      /* Shortcut: only one node left */
+      IGRAPH_CHECK(igraph_vector_push_back(&clique, VECTOR(new_cand)[0]));
+      IGRAPH_CHECK(func(&clique, data, &cont));
+      if (!cont) {
+        /* The callback function requested to stop the search */
+        break;
+      }
+      igraph_vector_pop_back(&clique);
+      igraph_vector_pop_back(&clique);
+      continue;
+    }
+
+    /* Find the next best candidate node in new_fini */
+    l = igraph_vector_size(&new_cand);
+    best_cand_degree = -1;
+    j = igraph_vector_size(&new_fini);
+    for (i = 0; i < j; i++) {
+      k = (long int)VECTOR(new_fini)[i];
+      IGRAPH_CHECK(igraph_vector_intersect_sorted(&new_cand, igraph_adjlist_get(&adj_list, k), &cn));
+      if (igraph_vector_size(&cn) > best_cand_degree) {
+        best_cand_degree = igraph_vector_size(&cn);
+        IGRAPH_CHECK(igraph_vector_update(&best_fini_cand_nbrs, &cn));
+        if (best_cand_degree == l) {
+          /* Cool, we surely have the best candidate node here as best_cand_degree can't get any better */
+          break;
+        }
+      }
+    }
+    /* Shortcut here: we don't have to examine new_cand */
+    if (best_cand_degree == l) {
+      igraph_vector_pop_back(&clique);
+      continue;
+    }
+    /* Still finding best candidate node */
+    best_fini_cand_degree = best_cand_degree;
+    best_cand_degree = -1;
+    j = igraph_vector_size(&new_cand);
+    l = l - 1;
+    for (i = 0; i < j; i++) {
+      k = (long int)VECTOR(new_cand)[i];
+      IGRAPH_CHECK(igraph_vector_intersect_sorted(&new_cand, igraph_adjlist_get(&adj_list, k), &cn));
+      if (igraph_vector_size(&cn) > best_cand_degree) {
+        best_cand_degree = igraph_vector_size(&cn);
+        IGRAPH_CHECK(igraph_vector_update(&best_cand_nbrs, &cn));
+        if (best_cand_degree == l) {
+          /* Cool, we surely have the best candidate node here as best_cand_degree can't get any better */
+          break;
+        }
+      }
+    }
+
+    /* Create a new stack frame in case we back out later */
+    new_frame_ptr = igraph_Calloc(1, igraph_i_maximal_cliques_stack_frame);
+    if (new_frame_ptr == 0) {
+      IGRAPH_ERROR("cannot allocate new stack frame", IGRAPH_ENOMEM);
+    }
+    IGRAPH_FINALLY(igraph_free, new_frame_ptr);
+    *new_frame_ptr = frame;
+    memset(&frame, 0, sizeof(frame));
+    IGRAPH_CHECK(igraph_stack_ptr_push(&stack, new_frame_ptr));
+    IGRAPH_FINALLY_CLEAN(1);  /* ownership of new_frame_ptr taken by the stack */
+    /* Ownership of the current frame and its vectors (frame.cand, frame.done, frame.cand_filtered)
+     * is taken by the stack from now on. Vectors in frame must be re-initialized with new_cand,
+     * new_fini and stuff. The old frame.cand and frame.fini won't be leaked because they are
+     * managed by the stack now. */
+    frame.cand = new_cand;
+    frame.fini = new_fini;
+    IGRAPH_CHECK(igraph_vector_init(&new_cand, 0));
+    IGRAPH_CHECK(igraph_vector_init(&new_fini, 0));
+    IGRAPH_CHECK(igraph_vector_init(&frame.cand_filtered, 0));
+
+    /* Adjust frame.cand_filtered */
+    if (best_cand_degree < best_fini_cand_degree) {
+      IGRAPH_CHECK(igraph_vector_difference_sorted(&frame.cand, &best_fini_cand_nbrs, &frame.cand_filtered));
+    } else {
+      IGRAPH_CHECK(igraph_vector_difference_sorted(&frame.cand, &best_cand_nbrs, &frame.cand_filtered));
+    }
+  }
+
+  IGRAPH_PROGRESS("Maximal cliques: ", 100.0, NULL);
+
+  igraph_adjlist_destroy(&adj_list);
+  igraph_vector_destroy(&clique);
+  igraph_vector_destroy(&new_cand);
+  igraph_vector_destroy(&new_fini);
+  igraph_vector_destroy(&cn);
+  igraph_vector_destroy(&best_cand_nbrs);
+  igraph_vector_destroy(&best_fini_cand_nbrs);
+  igraph_i_maximal_cliques_stack_frame_destroy(&frame);
+  igraph_i_maximal_cliques_stack_destroy(&stack);
+  IGRAPH_FINALLY_CLEAN(9);
+
+  return IGRAPH_SUCCESS;
 }
 
 int igraph_i_maximal_or_largest_cliques_or_indsets(const igraph_t *graph,
