@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2007  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2007-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard st, Cambridge, MA, 02138 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,7 +41,8 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
+// 02110-1301 USA
 //-----------------------------------------------------------------------------
 // Author   : Pascal Pons 
 // Email    : pons@liafa.jussieu.fr
@@ -51,12 +52,14 @@
 //-----------------------------------------------------------------------------
 // see readme.txt for more details
 
+/* FSF address above was fixed by Tamas Nepusz */
+
 
 #ifndef GRAPH_H
 #define GRAPH_H
 #include <iostream>
 
-#include "igraph.h"
+#include "igraph_community.h"
 
 using namespace std;
 
